PRO gimbalck, ckname, descr, scid, ckid, frame, ets, angles, axis, tol

   ;;
   ;; This example subroutine creates a new CK file with or appends to an
   ;; existing CK a Type 3 CK segment with attitude corresponding to the 
   ;; input gimbal rotation angles tagged with ephemeris time.
   ;;
   ;; Subroutine arguments -- all inputs -- are:
   ;;
   ;;    ckname    is the output CK file name.
   ;;
   ;;    descr     is a short (up to 40 characters) string describing 
   ;;              the input attitude data.
   ;;
   ;;    scid      is the spacecraft NAIF ID code.
   ;;    
   ;;    ckid      is the CK ID code.
   ;;
   ;;    frame     is the reference frame name.
   ;;
   ;;    ets       is the ordered array of ET times, as seconds past J2000,
   ;;              containing no duplicates.
   ;;
   ;;    angles    is the array of corresponding gimbal angles (in radians).
   ;;     
   ;;    axis      is the index of the axis about which the frame
   ;;              associated with the CK ID rotates by
   ;;              the angle (1=X, 2=Y, 3=Z).
   ;;
   ;;    tol       is the threshold, in seconds, that used to
   ;;              compute interpolation intervals.
   ;;
   ;; SPICE kernels needed to support time conversions and to resolve frame
   ;; names must be loaded into prior to calling this routine.
   ;;
   ;; Contact Boris Semenov (Boris.Semenov@jpl.nasa.gov) if you have any
   ;; question about this example.
   ;;
   ;; Version 1.0, September 27, 2006
   ;;

   ;;
   ;; Sanity check: ets and angles must have the same number of elements 
   ;; and this number must be non-zero.
   ;;
   if n_elements( ets ) NE n_elements( angles ) then begin
      print, 'gimbalck error: ets and angles have different dimensions.'
   endif
   n = n_elements( ets )

   if n LE 0 then begin
      print, 'gimbalck error: number of input points must be greater than 0.'
      return
   endif

   ;;
   ;; Create a 4xn matrix for quaternions, a 3xn matrix for angular 
   ;; velocities (AVs), a 1xn matrix for SCLKs and 1xn matrix for 
   ;; interpolation interval start times.
   ;;
   sclks = dblarr( n )
   quats = dblarr( 4, n )
   avs   = dblarr( 3, n )
   ints  = dblarr( n )

   ;;
   ;; Convert input ETs to SCLK ticks.
   ;;
   cspice_sce2c, scid, ets, sclks

   ;;
   ;; Loop over input data points to compute quaternions and AVs, 
   ;; and determine interval start times. Set interval index to 
   ;; zero before the loop.
   ;;
   intidx = 0

   for i = 0, n-1 do begin

      ;;
      ;; Construct the rotation matrix and quaternion corresponding to the 
      ;; current input rotation angle.
      ;;
      cspice_rotate, angles[i], axis, work_mat
      cspice_m2q, work_mat, work_quat
      quats[ (i*4):((i*4)+3) ] = work_quat

      ;;
      ;; For all points but the first compute av for the previous point and
      ;; determine whether the current point should be the start of an 
      ;; interval.
      ;;
      if i GT 0 then begin

         ;;
         ;; First, check if the current ET is greater than the previous ET.
         ;; If not, error out.
         ;;
         if ets[i] LE ets[i-1] then begin

            print, 'gimbalck error: out-of-order or duplicate ' + $
                   'input times.'
            return

         endif else begin

            ;;
            ;; Compute angular velocity of rotation from the 
            ;; previous point to the current point.
            ;;
            cspice_mtxm, work_mat, work_mat_prev, delta_mat            
            cspice_raxisa, delta_mat, axout, angout
            cspice_vscl, angout/(ets[i]-ets[i-1]), axout, work_av

            ;;
            ;; Check if this point is far enough from the previous point to 
            ;; be the start of the next interval. Assign av of the previous
            ;; point based on this decision. 
            ;;
            if ( ets[i]-ets[i-1] ) GT tol then begin

               ;;
               ;; This and the previous points are too far apart. Make
               ;; this point the start of the new interval.
               ;;
               ints[ intidx ] = sclks[i]
               ++ intidx

               ;;
               ;; Assign zero to the previous point AV.
               ;;
               avs[ ((i-1)*3):(((i-1)*3)+2) ] = [ 0.d, 0.d, 0.d ]

            endif else begin

               ;;
               ;; This and the previous points are close enough to 
               ;; allow interpolation. Assign the AV that we computed 
               ;; to the previous point AV.
               ;;
               avs[ ((i-1)*3):(((i-1)*3)+2) ] = work_av

            endelse

         endelse

      endif else begin

         ;;
         ;; The first point is always the start of the first interval.
         ;;
         ints[ intidx ] = sclks[i]
         ++ intidx

      endelse

      ;;
      ;; Save previous rotation matrix. It will be used in computing
      ;; av on the next step.
      ;;
      work_mat_prev = work_mat

   endfor

   ;;
   ;; Set the last point AV to zero.
   ;;
   avs[ ((n-1)*3):(((n-1)*3)+2) ] = [ 0.d, 0.d, 0.d ]

   ;;
   ;; Copy the portion of the intervals array that had been filled in to 
   ;; the array that will be provided to ckw03.
   ;;
   starts = ints[0:(intidx-1)]

   ;;
   ;; All data for calling ckw03 have been prepared. Depending on whether
   ;; the output CK exists or not open it for write access using ckopn or 
   ;; dafopw.
   ;;
   if ( cspice_exists( ckname ) ) then begin
      cspice_dafopw, ckname, handle
   endif else begin
      cspice_ckopn, ckname, descr, 0, handle
   endelse

   ;;
   ;; Call ckw03 to write all collected data in one shot.
   ;;
   cspice_ckw03, handle, sclks[0], sclks[n-1], ckid, frame, 1L, descr, $
                 sclks, quats, avs, starts

   ;;
   ;; All done. Close CK.
   ;;
   cspice_ckcls, handle

END
