;;
;; This simple script involkes gimbalck routine to create a sample CK file 
;; containing attitude for all four ROSETTA appendage gimbals -- +Y and 
;; -Y solar array gimbals and HGA EL and AZ gimbals.
;;
;; Contact Boris Semenov (Boris.Semenov@jpl.nasa.gov) if you have any
;; question about this example.
;;
;; Version 1.0, September 27, 2006
;;

PRO ros_gimbalck_example

   ;;
   ;; Load kernels. This meta-kernel must list LSK and ROSETTA SCLK (for 
   ;; time conversions) and ROSETTA FK (to make all needed frames known
   ;; to SPICE).
   ;;
   metakernel = 'ros_gimbalck_example.furnsh'
   cspice_furnsh, metakernel

   ;;
   ;; Assign gimbalck inputs applicable to all segments.
   ;;
   ;; ckname is the name of the ouput CK. The first call to gimbalck will
   ;; create this file while subsequent calls will append more segments to it.
   ;;
   ckname = 'ros_app_example.bc'
   if cspice_exists( ckname ) then begin
      print, 'Sample CK file ' + ckname + ' already exists. Cannot proceed.'
      return
   endif
  
   ;;
   ;; This descriptive string will be attached to each segment (as 
   ;; 'segment ID') and to the whole CK file (as 'internal file
   ;; name').
   ;;
   descr  = 'TEST ROSETTA APPENDAGE ORIENTATION'

   ;;
   ;; This is ROSETTA NAIF ID. It is used by gimbalck in calls to SCLK
   ;; routines converting from ET to SCLK ticks.
   ;;
   scid   = -226

   ;;
   ;; This is maximum gap between input data points over which the output
   ;; CK will allow interpolation, given in seconds.
   ;;
   tol    = 30.d

   ;;
   ;; The ets and angles arrays are filled in with some bogus values below.
   ;; In reality they will have to be filled with the actual solar array and
   ;; HGA gimbal telemetry data extracted from DDS.
   ;;
   utcs   = [ '2006 JUN 01 12:00:00', $
              '2006 JUN 01 12:00:10', $
              '2006 JUN 01 12:00:20', $
              '2006 JUN 01 12:01:00', $
              '2006 JUN 01 12:01:10'    ]

   cspice_str2et, utcs, ets

   angles = [ 0.d, 10.d, 20.d, 60.d, 70.d ] / cspice_dpr()

   ;;
   ;; The next four blocks call gimbalck with the general inputs set above
   ;; and specific inputs -- CK ID, reference frame name, and index of the 
   ;; gimbal rotation axis -- defined prior to each call. The IDs, frame
   ;; names and axes are consistent with the ROSETTA frame hierarchy defined 
   ;; in the frame kernel.
   ;;

   ;;
   ;; Assign gimbalck inputs specific for +Y solar array gimbal
   ;; and run gimbalck to make +Y SA CK segment.
   ;;
   ckid  = -226015
   frame = 'ROS_SA+Y_ZERO' 
   axis  = 2

   gimbalck, ckname, descr, scid, ckid, frame, ets, angles, axis, tol

   ;;
   ;; Assign gimbalck inputs specific for -Y solar array gimbal
   ;; and run gimbalck to make -Y SA CK segment.
   ;;
   ckid  = -226025
   frame = 'ROS_SA-Y_ZERO' 
   axis  = 2

   gimbalck, ckname, descr, scid, ckid, frame, ets, angles, axis, tol

   ;;
   ;; Assign gimbalck inputs specific for HGA EL gimbal
   ;; and run gimbalck to make HGA EL CK segment.
   ;;
   ckid  = -226071
   frame = 'ROS_SPACECRAFT' 
   axis  = 2

   gimbalck, ckname, descr, scid, ckid, frame, ets, angles, axis, tol

   ;;
   ;; Assign gimbalck inputs specific for HGA AZ gimbal
   ;; and run gimbalck to make HGA AZ CK segment.
   ;;
   ckid  = -226072
   frame = 'ROS_HGA_EL' 
   axis  = 3

   gimbalck, ckname, descr, scid, ckid, frame, ets, angles, axis, tol

   ;;
   ;; Unload kernels.
   ;;
   cspice_unload, metakernel

   ;;
   ;; Report that the file was created successfully.
   ;;
   if cspice_exists( ckname ) then begin
      print, 'Sample CK file ' + ckname + ' was created successfully.'
   endif

END

